@@ -26,6 +26,12 @@ public class FrameSearch {
 	
 	public static int findPossibleStarts_Forward(Gene cluster, StringBuffer contigSeq, int posAr, int tempStop){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("START FO")){
+				return FrameSearch.findPossibleStarts_Forward_AlternativeStarts(cluster, contigSeq, posAr, tempStop, GeneFinder.alternativeCodons.get("START FO"));
+			}
+		}
+		
 		if(tempStop < (int)Math.max(0,(cluster.startPos-GeneFinder.readLength))){
 			return -1;
 		}
@@ -69,6 +75,12 @@ public class FrameSearch {
 	
 	public static int findPossibleStarts_Reverse(Gene cluster, StringBuffer contigSeq, int posAr, int tempStop){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("START RE")){
+				return FrameSearch.findPossibleStarts_Reverse_AlternativeStarts(cluster, contigSeq, posAr, tempStop, GeneFinder.alternativeCodons.get("START RE"));
+			}
+		}
+		
 		int start_RE = -1;
 		
 		if(tempStop < (int)Math.max(0,(cluster.startPos-GeneFinder.readLength))){
@@ -112,6 +124,112 @@ public class FrameSearch {
 		return start_RE;
 	}
 	
+	/*
+	 * if alternative start and stop codons are given, perform a more general search also respecting those codons
+	 * 
+	 */
+	
+	public static int findPossibleStarts_Forward_AlternativeStarts(Gene cluster, StringBuffer contigSeq, int posAr, int tempStop,String[] alternativeStarts){
+		
+		int start1 = -1;
+		
+		if(tempStop < (int)Math.max(0,(cluster.startPos-GeneFinder.readLength))){
+			return -1;
+		}
+		
+		String startPart = contigSeq.substring((int)Math.max(0,(cluster.startPos-GeneFinder.readLength)), tempStop);
+		
+		int startSub_alt[] = new int[alternativeStarts.length];
+		
+		for(int i = 0; i<alternativeStarts.length;++i){
+			startSub_alt[i] = startPart.lastIndexOf(alternativeStarts[i]);
+		}
+		
+		java.util.Arrays.sort(startSub_alt);
+		
+		for(int i = startSub_alt.length -1; i>= 0;i--){
+			if(startSub_alt[i] > -1){ 
+				start1 = (int) Math.max(0,(cluster.startPos-GeneFinder.readLength)) + startSub_alt[i]; 
+				break;
+			}
+		}
+		
+		if(start1 == -1){
+			return start1;
+		}
+		
+		boolean foundSameFrame = false;
+		
+		for(int i = 0; i<posAr;++i){
+			if((cluster.possibleStarts_Forward[i] - start1) % 3 == 0){
+				foundSameFrame = true;
+				break;
+			}
+		}
+		
+		if(!foundSameFrame){
+			cluster.possibleStarts_Forward[posAr++] = start1;
+			findPossibleStarts_Forward_AlternativeStarts(cluster,contigSeq,posAr,start1,alternativeStarts);
+		}else{
+			findPossibleStarts_Forward_AlternativeStarts(cluster,contigSeq,posAr,start1,alternativeStarts);
+		}
+		
+		return start1;
+	}
+
+	/*
+	 * if alternative start and stop codons are given, perform a more general search also respecting those codons
+	 * 
+	 */
+	
+	public static int findPossibleStarts_Reverse_AlternativeStarts(Gene cluster, StringBuffer contigSeq, int posAr, int tempStop,String[] alternativeStops){
+	
+	int start_RE = -1;
+	
+	if(tempStop < (int)Math.max(0,(cluster.startPos-GeneFinder.readLength))){
+		return -1;
+	}
+	
+	String startPart = contigSeq.substring((int)Math.max(0,(cluster.startPos-GeneFinder.readLength)), tempStop);
+	
+	int startSub_alt[] = new int[alternativeStops.length];
+	
+	for(int i = 0; i<alternativeStops.length;++i){
+		startSub_alt[i] = startPart.lastIndexOf(alternativeStops[i]);
+	}
+	
+	java.util.Arrays.sort(startSub_alt);
+	
+	for(int i = startSub_alt.length -1; i>= 0;i--){
+		if(startSub_alt[i] > -1){ 
+			start_RE = (int) Math.max(0,(cluster.startPos-GeneFinder.readLength)) + startSub_alt[i]; 
+			break;
+		}
+	}
+	
+	if(start_RE == -1){
+		return start_RE;
+	}
+	
+	boolean foundSameFrame = false;
+	
+	for(int i = 0; i<posAr;++i){
+		if((cluster.possibleStarts_Reverse[i] - start_RE) % 3 == 0){
+			foundSameFrame = true;
+			break;
+		}
+	}
+	
+	if(!foundSameFrame){
+		cluster.possibleStarts_Reverse[posAr++] = start_RE;
+		findPossibleStarts_Reverse_AlternativeStarts(cluster,contigSeq,posAr,start_RE,alternativeStops);
+	}else{
+		findPossibleStarts_Reverse_AlternativeStarts(cluster,contigSeq,posAr,start_RE,alternativeStops);
+	}
+	
+	return start_RE;
+}
+	
 	/*
 	 * new way of gene extraction by remembering all starts that are not in the same frame (maxNum = 3)
 	 * after that, starts and stops are checked if we find a combination that defines the frame of the cluster
@@ -122,6 +240,12 @@ public class FrameSearch {
 	
 	public static int findPossibleStops_Forward(Gene cluster, StringBuffer contigSeq, int posAr, int tempStart){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("STOP FO")){
+				return FrameSearch.findPossibleStops_Forward_AlternativeStops(cluster, contigSeq, posAr, tempStart, GeneFinder.alternativeCodons.get("STOP FO"));
+			}
+		}
+		
 		int stop_FO = -1;
 		
 		if(tempStart > (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1)){
@@ -174,6 +298,12 @@ public class FrameSearch {
 	
 	public static int findPossibleStops_Reverse(Gene cluster, StringBuffer contigSeq, int posAr, int tempStart){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("STOP RE")){
+				return FrameSearch.findPossibleStops_Reverse_AlternativeStop(cluster, contigSeq, posAr, tempStart, GeneFinder.alternativeCodons.get("STOP RE"));
+			}
+		}
+		
 		if(tempStart > (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1)){
 			return -1;
 		}
@@ -205,6 +335,112 @@ public class FrameSearch {
 		return start1;
 	}
 	
+	/*
+	 * if alternative start and stop codons are given, perform a more general search also respecting those codons
+	 * 
+	 */
+	
+	public static int findPossibleStops_Forward_AlternativeStops(Gene cluster, StringBuffer contigSeq, int posAr, int tempStart, String[] alternativeStops){
+		
+		int stop_FO = -1;
+		
+		if(tempStart > (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1)){
+			return -1;
+		}
+		
+		String stopPart = contigSeq.substring(tempStart, (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1));
+		
+		int stopSub_alt[] = new int[alternativeStops.length];
+		
+		for(int i = 0; i<alternativeStops.length;++i){
+			stopSub_alt[i] = stopPart.indexOf(alternativeStops[i]);
+		}
+		
+		java.util.Arrays.sort(stopSub_alt);
+		
+		for(int i = 0; i < stopSub_alt.length;++i){
+			if(stopSub_alt[i] > -1){ 
+				stop_FO = tempStart + stopSub_alt[i]; 
+				break;
+			}
+		}
+		
+		if(stop_FO == -1){
+			return stop_FO;
+		}
+		
+		boolean foundSameFrame = false;
+		
+		for(int i = 0; i<posAr;++i){
+			if((cluster.possibleStops_Forward[i] - stop_FO) % 3 == 0){
+				foundSameFrame = true;
+				break;
+			}
+		}
+		
+		if(!foundSameFrame){
+			cluster.possibleStops_Forward[posAr++] = stop_FO;
+			findPossibleStops_Forward_AlternativeStops(cluster,contigSeq,posAr,stop_FO+3,alternativeStops);
+		}else{
+			findPossibleStops_Forward_AlternativeStops(cluster,contigSeq,posAr,stop_FO+3,alternativeStops);
+		}
+		
+		return stop_FO;
+	}
+
+	/*
+	 * if alternative start and stop codons are given, perform a more general search also respecting those codons
+	 * 
+	 */
+	
+	public static int findPossibleStops_Reverse_AlternativeStop(Gene cluster, StringBuffer contigSeq, int posAr, int tempStart, String[] alternativeStarts){
+		
+		int start1 = -1;
+		
+		if(tempStart > (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1)){
+			return -1;
+		}
+		
+		String stopPart = contigSeq.substring(tempStart, (int) Math.min(contigSeq.length(),cluster.stopPos-2 + GeneFinder.readLength + 1));
+		
+		int stopSub_alt[] = new int[alternativeStarts.length];
+		
+		for(int i = 0; i<alternativeStarts.length;++i){
+			stopSub_alt[i] = stopPart.indexOf(alternativeStarts[i]);
+		}
+		
+		java.util.Arrays.sort(stopSub_alt);
+		
+		for(int i = 0; i < stopSub_alt.length;++i){
+			if(stopSub_alt[i] > -1){ 
+				start1 = tempStart + stopSub_alt[i]; 
+				break;
+			}
+		}
+		
+		if(start1 == -1){
+			return start1;
+		}
+		
+		boolean foundSameFrame = false;
+		
+		for(int i = 0; i<posAr;++i){
+			if((cluster.possibleStops_Reverse[i] - start1) % 3 == 0){
+				foundSameFrame = true;
+				break;
+			}
+		}
+		
+		if(!foundSameFrame){
+			cluster.possibleStops_Reverse[posAr++] = start1;
+			findPossibleStops_Reverse_AlternativeStop(cluster,contigSeq,posAr,start1+3,alternativeStarts);
+		}else{
+			findPossibleStops_Reverse_AlternativeStop(cluster,contigSeq,posAr,start1+3,alternativeStarts);
+		}
+		
+		return start1;
+	}
+
 	/*
 	 * test if there is one of the possible start-stop codon pairs which is in frame
 	 * take the smallest interval possible
