@@ -56,6 +56,9 @@ public class ProkaryoteExtraction {
 					if(line.startsWith(">")){
 						// test if correct contig
 						if(line.substring(1).startsWith(contigName)){
+							if(!((line.substring(1).startsWith(contigName+" ")) || (line.substring(1).length() == contigName.length()))){
+								continue;			// as an additional check to avoid picking the wrong contig because of name sub-similarities 											
+							}
 							// found right one, now extract sequence
 							while(((line = br.readLine()) != null) && (line.length() != 0) &&  (!(line.startsWith(">")))){
 								String line2 = "";
