@@ -234,6 +234,12 @@ public class Prokaryote_Specials {
 	
 	public static Vector<int[]> searchFO_orfs(String inputSeq){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("START FO")){
+				return searchFO_orfs_alternativeCodons(inputSeq, GeneFinder.alternativeCodons.get("START FO"), GeneFinder.alternativeCodons.get("STOP FO"));
+			}
+		}
+		
 		Vector<int[]> allORFs_FO = new Vector<int[]>();
 		
 		int foundNewATG = 1;
@@ -314,6 +320,106 @@ public class Prokaryote_Specials {
 		return allORFs_FO;
 	}
 	
+	/*
+	 * if alternative start and stop codons are specified, respect this in a more general orf search
+	 * 
+	 */
+	
+	public static Vector<int[]> searchFO_orfs_alternativeCodons(String inputSeq, String[] alternativeStarts_FO, String[] alternativeStops_FO){
+		
+		Vector<int[]> allORFs_FO = new Vector<int[]>();
+		
+		int foundNewATG = 1;
+		int posLastATG = 0;
+		
+		do{
+			
+			int startPos = -1;
+			
+			String startPart_alt = inputSeq.substring(posLastATG);
+			
+			int startSub_alt[] = new int[alternativeStarts_FO.length];
+			
+			for(int i = 0; i<alternativeStarts_FO.length;++i){
+				startSub_alt[i] = startPart_alt.indexOf(alternativeStarts_FO[i]);
+			}
+			
+			java.util.Arrays.sort(startSub_alt);
+			
+			for(int i = 0; i < startSub_alt.length;++i){
+				if(startSub_alt[i] > -1){ 
+					startPos = startSub_alt[i]; 
+					break;
+				}
+			}
+					
+			int stopPos = -1;
+			
+			int posLastStart = -1;
+			
+			if(startPos == -1){
+				foundNewATG = 0;
+				break;
+			}else{
+				startPos = startPos  + posLastATG;
+				posLastATG = startPos + 3;
+				posLastStart = startPos + 3;
+			}
+
+			int goOn = 0;
+			
+			do{
+				goOn = 0;
+				
+				String stopPart = inputSeq.substring(posLastStart);
+				
+				int stopSub[] = new int[alternativeStops_FO.length];
+				
+				for(int i = 0; i<alternativeStops_FO.length;++i){
+					stopSub[i] = stopPart.indexOf(alternativeStops_FO[i]);
+				}
+				
+				java.util.Arrays.sort(stopSub);
+				
+				for(int i = 0; i < stopSub.length;++i){
+					if(stopSub[i] > -1){ 
+						if(((((posLastStart + stopSub[i])-startPos) % 3) == 0)){
+							stopPos = posLastStart + stopSub[i]; 
+						}else{
+							posLastStart = posLastStart + stopSub[i]+1;
+							goOn = 1;
+						}
+						break;
+					}
+				}
+				
+				if(stopPos != -1){
+					
+					if(stopPos-startPos > 30){
+						if(!checkIfORFcovered(allORFs_FO,new int[]{startPos,(stopPos+2)})){
+							allORFs_FO.add(new int[]{startPos,(stopPos+2)});
+							for(int i=startPos;i<=stopPos+2;++i){
+								cov[i]++;
+							}
+						}else{
+							alreadyCovered++;
+						}						
+					}else{
+						notCounted++;
+					}
+					
+					break;
+				}
+				
+			}while(goOn == 1);
+			
+			
+		}while(foundNewATG == 1);
+		
+		
+		return allORFs_FO;
+	}
+	
 	/*
 	 * searches all ORFs assuming reverse direction
 	 * note: no length limit is set, ORFs too short should be penalized in the BIC scoring
@@ -321,6 +427,12 @@ public class Prokaryote_Specials {
 	
 	public static Vector<int[]> searchRE_orfs(String inputSeq){
 		
+		if(!GeneFinder.alternativeCodons.isEmpty()){
+			if(GeneFinder.alternativeCodons.containsKey("START RE")){
+				return searchRE_orfs_alternativeCodons(inputSeq, GeneFinder.alternativeCodons.get("STOP RE"), GeneFinder.alternativeCodons.get("START RE")); // are stored the other way around so start is stop and vice versa
+			}
+		}
+		
 		Vector<int[]> allORFs_RE= new Vector<int[]>();
 		
 		int foundNewCAT = 1;
@@ -401,6 +513,105 @@ public class Prokaryote_Specials {
 		return allORFs_RE;
 	}
 	
+	/*
+	 * if alternative start and stop codons are specified, respect this in a more general orf search
+	 * 
+	 */
+	
+	public static Vector<int[]> searchRE_orfs_alternativeCodons(String inputSeq, String[] alternativeStarts_RE, String[] alternativeStops_RE){
+		
+		Vector<int[]> allORFs_RE= new Vector<int[]>();
+		
+		int foundNewCAT = 1;
+		int posLastCAT = inputSeq.length();
+		
+		do{
+			int startPos = -1;
+			
+			String startPart_alt = inputSeq.substring(0,posLastCAT);
+			
+			int startSub_alt[] = new int[alternativeStarts_RE.length];
+			
+			for(int i = 0; i<alternativeStarts_RE.length;++i){
+				startSub_alt[i] = startPart_alt.lastIndexOf(alternativeStarts_RE[i]);
+			}
+			
+			java.util.Arrays.sort(startSub_alt);
+			
+			for(int i = startSub_alt.length -1; i>= 0;i--){
+				if(startSub_alt[i] > -1){ 
+					startPos = startSub_alt[i]; 
+					break;
+				}
+			}
+			
+			int stopPos = -1;
+			
+			int posLastStop = -1;
+			
+			if(startPos == -1){
+				foundNewCAT = 0;
+				break;
+			}else{
+				posLastCAT = startPos;
+				posLastStop = startPos;
+			}
+			
+			int goOn = 0;
+			
+			do{
+				goOn = 0;
+				
+				String stopPart = inputSeq.substring(0,posLastStop);
+			
+				int stopSub[] = new int[alternativeStops_RE.length];
+				
+				for(int i = 0; i<alternativeStops_RE.length;++i){
+					stopSub[i] = stopPart.lastIndexOf(alternativeStops_RE[i]);
+				}
+				
+				java.util.Arrays.sort(stopSub);
+				
+				for(int i = stopSub.length -1; i>= 0;i--){
+					if(stopSub[i] > -1){ 
+						if(((startPos-stopSub[i]) % 3) == 0){
+							stopPos = stopSub[i];
+						}else{
+							posLastStop = stopSub[i]+2;
+							goOn = 1;
+						}							
+						break;
+					}
+				}
+				
+				if(stopPos != -1){
+					
+					if(startPos-stopPos > 30){
+						if(!checkIfORFcovered(allORFs_RE,new int[]{stopPos,(startPos+2)})){
+							allORFs_RE.add(new int[]{stopPos,(startPos+2)});
+							for(int i=stopPos;i<=startPos+2;++i){
+								cov[i]++;
+							}
+						}else{
+							alreadyCovered++;
+						}					
+					}else{
+						notCounted++;
+					}
+					
+					
+					break;
+				}
+				
+			}while(goOn == 1);
+			
+			
+		}while(foundNewCAT == 1);
+		
+		
+		return allORFs_RE;
+	}
+	
 	/*
 	 * filter out all orfs that are completely included in bigger ones
 	 */
