@@ -58,6 +58,7 @@ public class ReadInParameters_GeneFinder {
 		boolean foundProkaryote = false;
 		boolean foundSequential = false;
 		boolean foundInprogea = false;
+		boolean foundAlternativeCodons = false;
 		
 		
 		if(!parameter.isEmpty() && args.length > 0){
@@ -230,6 +231,12 @@ public class ReadInParameters_GeneFinder {
 						inputText += "minimal interval length: " + GeneFinder.interval + "\n";	
 					}
 						
+				} else if(arg.equals("-altCodon")){  // alternative start and stop codons
+					String pathToAlternative = args[i+1];
+					readInAlternativeStartsStops(pathToAlternative);
+					foundAlternativeCodons = true;
+					inputText += "Alternative Starts and stops provided \n";
+				
 				} else if(arg.equals("-noAmbiOpti")){  // turn on or off the optimization of ambiguous reads
 					foundAmbiOpti = true;			
 					GeneFinder.noAmbiOpti = true;
@@ -321,6 +328,21 @@ public class ReadInParameters_GeneFinder {
 		
 		if(!havePathOut){
 			GeneFinder.pathOut = "";
+		}else{
+			// check if directory exists, if not, create it
+			File f = new File(GeneFinder.pathOut);
+			if(!f.exists()){
+				Runtime rtAlign = Runtime.getRuntime();
+				try {
+					String exe = "mkdir " + GeneFinder.pathOut;
+					Process pc = rtAlign.exec(exe);
+					pc.waitFor();			
+				} catch (IOException e) {
+					e.printStackTrace();
+				} catch (InterruptedException e) {
+					e.printStackTrace();
+				}
+			}
 		}
 		if(!foundGenome){
 			System.out.println("No genome file specified. Use \"-h\" to print usage options. ");
@@ -410,6 +432,7 @@ public class ReadInParameters_GeneFinder {
 			GeneFinder.inprogeaCall = false;
 		}
 		
+		
 		GeneFinder.logFile = new File(GeneFinder.pathOut+"log_it" + GeneFinder.iteration + ".txt");
 		if(!GeneFinder.secondPart){
 			System.out.println(inputText);
@@ -437,6 +460,66 @@ public class ReadInParameters_GeneFinder {
 
 	}	
 	
+	/*
+	 * reads in the alternative start and stop codons from a given input file
+	 * one line per codon type, with codons tab separated
+	 */
+	
+	public static void readInAlternativeStartsStops(String altFile) {
+		
+		Map<String,String[]> altCodons = new HashMap<String,String[]>();
+			
+		try {
+			
+			BufferedReader br = new BufferedReader(new FileReader(altFile));
+			
+			String line = "";
+			
+			while((line = br.readLine()) != null){
+				
+				String[] lineArr = line.split("\t");
+				String[] temp = new String[lineArr.length-1];
+				
+				if(line.startsWith("START FO")){
+
+					for(int i = 1; i<lineArr.length;++i){
+						temp[i-1] = lineArr[i];
+					}
+					
+					altCodons.put("START FO",temp);
+				}
+				if(line.startsWith("START RE")){
+					for(int i = 1; i<lineArr.length;++i){
+						temp[i-1] = lineArr[i];
+					}
+					
+					altCodons.put("STOP RE",temp);	// for GIIRA start and stops are switched for the reverse direction
+				}
+				if(line.startsWith("STOP FO")){
+					for(int i = 1; i<lineArr.length;++i){
+						temp[i-1] = lineArr[i];
+					}
+					
+					altCodons.put("STOP FO",temp);
+				}
+				if(line.startsWith("STOP RE")){
+					for(int i = 1; i<lineArr.length;++i){
+						temp[i-1] = lineArr[i];
+					}
+					
+					altCodons.put("START RE",temp);	// for GIIRA start and stops are switched for the reverse direction
+				}
+			}
+			
+		} catch (FileNotFoundException e) {
+			e.printStackTrace();
+		} catch (IOException e) {
+			e.printStackTrace();
+		}
+		
+		GeneFinder.alternativeCodons = altCodons;
+	}
+	
 	/*
 	 * print the help text to screen
 	 */
@@ -457,7 +540,7 @@ public class ReadInParameters_GeneFinder {
 				" \n -iG [pathToGenomes] : specify path to directory with genome files in fasta format \n" +
 				" \n -iR [pathToRna] : specify path to directory with rna read files in fastq format \n" +
 				" \n -scripts [absolutePath] : specify the absolute path to the directory containing the required helper scripts, DEFAULT: directory of GIIRA.jar \n" +
-				" \n -out [pathToResults] : specify the directory that shall contain the results files \n" +
+				" \n -out [pathToResults] : specify the absolute pyth to the directory that shall contain the results files \n" +
 				" \n -outName [outputName] : specify desired name for output files, DEFAULT: genes \n" +
 				" \n -haveSam [samfileName]: if a sam file already exists, provide the name, else a mapping is performed. NOTE: the sam file has to be sorted according to read names! \n" +
 				" \n -nT [numberThreads] : specify the maximal number of threads that are allowed to be used, DEFAULT: 1 \n" +
@@ -468,7 +551,7 @@ public class ReadInParameters_GeneFinder {
 				//" \n -splitRunAndOpti [y/n] : indicates if the optimization and giira shall be run separately, to reduce the memory consumption (y), DEFAULT: n" +
 				" \n -mem [int] : specify the amount of memory that cplex is allowed to use \n" +
 				" \n -maxReportedHits [int] : if using BWA as mapping tool, specify the maximal number of reported hits, DEFAULT: 2 \n" +
-				" \n -prokaryote : if specified, genome is treated as prokaryotic, no spliced reads are accepted, and structural genes are resolved. DEFAULT: n \n" +
+				" \n -prokaryote : if specified, genome is treated as prokaryotic, no spliced reads are accepted, and structural genes are resolved. DEFAULT: False \n" +
 				" \n -minCov [double] : specify the minimum required coverage of the gene candidate extraction, DEFAULT: -1 (is estimated from mapping) \n" +
 				" \n -maxCov [double] : optional maximal coverage threshold, can also be estimated from mapping (DEFAULT) \n" +
 				" \n -endCov [double] : if the coverage falls below this value, the currently open candidate gene is closed. This value can be estimated from the minimum coverage (-1); DEFAULT: -1 \n" +
@@ -476,6 +559,7 @@ public class ReadInParameters_GeneFinder {
 				" \n -interval [int] : specify the minimal size of an interval between near candidate genes, if \"-1\" it equals the read length. DEFAULT: -1 \n " +
 				" \n -splLim [double] : specify the minimal coverage that is required to accept a splice site, if (-1) the threshold is equal to minCov, DEFAULT: -1 \n" +
 				" \n -rL [int] : specify read length, otherwise this information is extracted from SAM file (DEFAULT) \n" +
+				" \n -altCodon [pathToAlternativeCodons] : specify path to txt file with alternative start and stop codons, see example file in scripts folder \n" +
 				" \n -samForSequential [pathToSamFile] : if it is desired to analyse chromosomes in a sequential manner, provide a chromosome sorted sam file in addition to the one sorted by read names, DEFAULT: noSequential \n" +
 				" \n -noAmbiOpti : if specified, ambiguous hits are not included in the analysis \n" +					
 				" \n -settingMapper [(list of parameters)] : A comma-separated list of the desired parameters for TopHat or BWA. Please provide \n" +
