@@ -138,7 +138,21 @@ public class SamParser {
 
 							totalHitCount++;
 							
-							if(!parts[0].equals(currentReadID)){  // now we have proceeded to a new read
+							String adaptedName = "";
+							
+							if(parts[0].contains(":")){
+								String[] nameParts = parts[0].split(":");
+								for(int i=0;i<nameParts.length;++i){
+									adaptedName += nameParts[i] + ";;;";   // necessary to avoid cplex or glpk errors
+								}
+								
+								adaptedName = adaptedName.substring(0,(adaptedName.length()-3));
+							}else{
+								adaptedName = parts[0];
+							}
+							
+							
+							if(!adaptedName.equals(currentReadID)){  // now we have proceeded to a new read
 
 								if(GeneFinder.iteration == 2 && currentRead != null && currentRead.isMulti == 1){
 									
@@ -147,7 +161,20 @@ public class SamParser {
 										do{
 											String[] partsReaSam = lineReaSam.split("	");										
 											
-											if(currentRead.rnaID.equals(partsReaSam[0])){
+											String adaptedNameReaSam = "";
+											
+											if(partsReaSam[0].contains(":")){
+												String[] nameParts = partsReaSam[0].split(":");
+												for(int i=0;i<nameParts.length;++i){
+													adaptedNameReaSam += nameParts[i] + ";;;";   // necessary to avoid cplex or glpk errors
+												}
+												
+												adaptedNameReaSam = adaptedNameReaSam.substring(0,(adaptedNameReaSam.length()-3));
+											}else{
+												adaptedNameReaSam = partsReaSam[0];
+											}
+											
+											if(currentRead.rnaID.equals(adaptedNameReaSam)){
 												allReassigned.put(Integer.parseInt(partsReaSam[3]),partsReaSam[2]);
 											}else{
 												break;
@@ -165,12 +192,12 @@ public class SamParser {
 									
 								}
 
-								currentReadID = parts[0];				
+								currentReadID = adaptedName;				
 
 								// set up new rna node
 
 								Rna newRna = new Rna();
-								newRna.rnaID = parts[0];  		
+								newRna.rnaID = adaptedName;  		
 								newRna.isMulti = 0;
 								newRna.hitNum = 1;
 								newRna.assignedNum = 0;
@@ -479,9 +506,22 @@ public class SamParser {
 							do{
 								String[] partsReaSam = lineReaSam.split("	");
 								
-								if(partsReaSam[0].compareTo(currentRead.rnaID) > 0){
+								String adaptedNameReaSam = "";
+								
+								if(partsReaSam[0].contains(":")){
+									String[] nameParts = partsReaSam[0].split(":");
+									for(int i=0;i<nameParts.length;++i){
+										adaptedNameReaSam += nameParts[i] + ";;;";   // necessary to avoid cplex or glpk errors
+									}
+									
+									adaptedNameReaSam = adaptedNameReaSam.substring(0,(adaptedNameReaSam.length()-3));
+								}else{
+									adaptedNameReaSam = partsReaSam[0];
+								}
+								
+								if(adaptedNameReaSam.compareTo(currentRead.rnaID) > 0){
 									break;  // we exceeded this read, so stop
-								}else if(currentRead.rnaID.equals(partsReaSam[0])){
+								}else if(currentRead.rnaID.equals(adaptedNameReaSam)){
 									allReassigned.put(Integer.parseInt(partsReaSam[3]),partsReaSam[2]);
 								}
 							}while((lineReaSam = br.readLine()) != null);
@@ -953,9 +993,23 @@ public class SamParser {
 							
 							Rna read;
 							
-							if(seenReads.keySet().contains(parts[0])){
+							String adaptedName = "";
+							
+							if(parts[0].contains(":")){
+								String[] nameParts = parts[0].split(":");
+								for(int i=0;i<nameParts.length;++i){
+									adaptedName += nameParts[i] + ";;;";   // necessary to avoid cplex or glpk errors
+								}
 								
-								Vector<Object> temp = seenReads.get(parts[0]);
+								adaptedName = adaptedName.substring(0,(adaptedName.length()-3));
+							}else{
+								adaptedName = parts[0];
+							}
+							
+							
+							if(seenReads.keySet().contains(adaptedName)){
+								
+								Vector<Object> temp = seenReads.get(adaptedName);
 		
 								if(((Integer)temp.get(0)) != 0){
 									
@@ -973,7 +1027,7 @@ public class SamParser {
 											
 											temp.clear();
 											temp.add(0);
-											seenReads.put(parts[0],temp);
+											seenReads.put(adaptedName,temp);
 											
 											if(totalHitCount % 100000 == 0){
 		
@@ -1005,7 +1059,7 @@ public class SamParser {
 										interChromoTotalCount++;
 										Vector<Object> temp = new Vector<Object>();
 										temp.add(0);									
-										seenReads.put(parts[0],temp);
+										seenReads.put(adaptedName,temp);
 										interChromoTotalCount++;
 										break;
 									}
@@ -1049,7 +1103,7 @@ public class SamParser {
 									Vector<Object> temp = new Vector<Object>();
 									temp.add(1);
 									temp.add(read);
-									seenReads.put(parts[0],temp);	
+									seenReads.put(adaptedName,temp);	
 								}
 							}
 
