@@ -17,7 +17,7 @@ public class Rna {
 	
 	public double quality; 	  	
 
-	public Vector<Object[]> contigsMappedOn = new Vector<Object[]>(); // contains several Arrays á: [contig, alignPos, cigarString, mapQual,spliceInfo,mismatchInfo,direcInfo] (one for each hit)
+	public Vector<Object[]> contigsMappedOn = new Vector<Object[]>(); // contains several Arrays ala: [contig, alignPos, cigarString, mapQual,spliceInfo,mismatchInfo,direcInfo] (one for each hit)
 	
 	public int isMulti;		// indicator if this read is an ambiguous read
 	
